@@ -24,27 +24,28 @@ See https://tools.ietf.org/html/rfc4331
 from xandikos import webdav
 
 
-class QuotaAvailableBytesProperty(webdav.Property):
-    """quota-available-bytes
-    """
+FEATURE: str = "quota"
 
-    name = '{DAV:}quota-available-bytes'
+
+class QuotaAvailableBytesProperty(webdav.Property):
+    """quota-available-bytes"""
+
+    name = "{DAV:}quota-available-bytes"
     resource_type = None
     in_allprops = False
     live = True
 
-    def get_value(self, href, resource, el):
+    async def get_value(self, href, resource, el, environ):
         el.text = resource.get_quota_available_bytes()
 
 
 class QuotaUsedBytesProperty(webdav.Property):
-    """quota-used-bytes
-    """
+    """quota-used-bytes"""
 
-    name = '{DAV:}quota-used-bytes'
+    name = "{DAV:}quota-used-bytes"
     resource_type = None
     in_allprops = False
     live = True
 
-    def get_value(self, href, resource, el):
+    async def get_value(self, href, resource, el, environ):
         el.text = resource.get_quota_used_bytes()
