@@ -20,9 +20,10 @@
 """WSGI wrapper for xandikos.
 """
 
-import os
 import posixpath
 
+from .web import WELLKNOWN_DAV_PATHS
+
 
 class WellknownRedirector(object):
     """Redirect paths under .well-known/ to the appropriate paths."""
@@ -33,10 +34,8 @@ class WellknownRedirector(object):
 
     def __call__(self, environ, start_response):
         # See https://tools.ietf.org/html/rfc6764
-        path = posixpath.normpath(
-            environ['SCRIPT_NAME'] + environ['PATH_INFO'])
+        path = posixpath.normpath(environ["SCRIPT_NAME"] + environ["PATH_INFO"])
         if path in WELLKNOWN_DAV_PATHS:
-            start_response('302 Found', [
-                ('Location', self._dav_root)])
+            start_response("302 Found", [("Location", self._dav_root)])
             return []
         return self._inner_app(environ, start_response)
